@@ -534,7 +534,7 @@ static int write_changes(struct sfdisk *sf)
 			rc = move_partition_data(sf, sf->partno, sf->orig_pa);
 		if (!rc) {
 			fdisk_info(sf->cxt, _("\nThe partition table has been altered."));
-			rc = fdisk_reread_partition_table(sf->cxt);
+			fdisk_reread_partition_table(sf->cxt);
 		}
 	}
 	if (!rc)
